# Summary

- [1. Kubernetes 简介](introduction/index.md)
  - [1.1 基本概念](introduction/concepts.md)
  - [1.2 Kubernetes 101](introduction/101.md)
  - [1.3 Kubernetes 201](introduction/201.md)
  - [1.4 Kubernetes 集群](introduction/cluster.md)

## 核心原理

- [2. 核心原理](architecture/index.md)
  - [2.1 架构原理](architecture/architecture.md)
  - [2.2 设计理念](architecture/concepts.md)
  - [2.3 核心组件](components/index.md)
  - [2.4 etcd](components/etcd.md)
  - [2.5 kube-apiserver](components/apiserver.md)
    - [访问 API](components/api-access.md)
    - [扩展 API](components/api-aggregation.md)
  - [2.6 kube-scheduler](components/scheduler.md)
  - [2.7 kube-controller-manager](components/controller-manager.md)
  - [2.8 kubelet](components/kubelet.md)
    - [容器运行时](components/container-runtime.md)
  - [2.9 kube-proxy](components/kube-proxy.md)
  - [2.10 kube-dns](components/kube-dns.md)
  - [2.11 Federation](components/federation.md)
  - [2.12 kubeadm](components/kubeadm.md)
  - [2.13 hyperkube](components/hyperkube.md)
  - [2.14 kubectl](components/kubectl.md)
- [3. 资源对象](concepts/index.md)
  - [Autoscaling](concepts/autoscaling.md)
  - [ConfigMap](concepts/configmap.md)
  - [CronJob](concepts/cronjob.md)
  - [CustomResourceDefinition](concepts/customresourcedefinition.md)
  - [DaemonSet](concepts/daemonset.md)
  - [Deployment](concepts/deployment.md)
  - [Ingress](concepts/ingress.md)
  - [Job](concepts/job.md)
  - [LocalVolume](concepts/local-volume.md)
  - [Namespace](concepts/namespace.md)
  - [NetworkPolicy](concepts/network-policy.md)
  - [Node](concepts/node.md)
  - [PersistentVolume](concepts/persistent-volume.md)
  - [Pod](concepts/pod.md)
  - [PodPreset](concepts/podpreset.md)
  - [ReplicaSet](concepts/replicaset.md)
  - [Resource Quota](concepts/quota.md)
  - [Secret](concepts/secret.md)
  - [SecurityContext](concepts/security-context.md)
  - [Service](concepts/service.md)
  - [ServiceAccount](concepts/serviceaccount.md)
  - [StatefulSet](concepts/statefulset.md)
  - [ThirdPartyResources](concepts/thirdpartyresources.md)
  - [Volume](concepts/volume.md)

## 部署配置

- [4. 部署配置](deploy/index.md)
  - [4.1 kubectl 客户端](deploy/kubectl.md)
  - [4.2 单机部署](deploy/single.md)
  - [4.3 集群部署](deploy/cluster.md)
    - [kubeadm](deploy/kubeadm.md)
    - [kops](deploy/kops.md)
    - [Kubespray](deploy/kubespray.md)
    - [Azure](deploy/azure.md)
    - [Windows](deploy/windows.md)
    - [LinuxKit](deploy/k8s-linuxkit.md)
    - [Frakti](deploy/frakti/index.md)
    - [kubeasz](https://github.com/gjmzj/kubeasz)
    - [特性开关](deploy/feature-gates.md)
    - [最佳配置](deploy/kubernetes-configuration-best-practice.md)
- [5. 附加组件](addons/index.md)
  - [5.1 Dashboard](addons/dashboard.md)
  - [5.2 监控](addons/monitor.md)
  - [5.3 日志](addons/logging.md)
  - [5.4 Metrics](addons/metrics.md)
  - [5.5 Cluster AutoScaler](addons/cluster-autoscaler.md)
- [6. Kubernetes-The-Hard-Way](deploy/kubernetes-the-hard-way/index.md)
  - [6.1 准备部署环境](deploy/kubernetes-the-hard-way/01-prerequisites.md)
  - [6.2 安装必要工具](deploy/kubernetes-the-hard-way/02-client-tools.md)
  - [6.3 创建计算资源](deploy/kubernetes-the-hard-way/03-compute-resources.md)
  - [6.4 配置创建证书](deploy/kubernetes-the-hard-way/04-certificate-authority.md)
  - [6.5 配置生成配置](deploy/kubernetes-the-hard-way/05-kubernetes-configuration-files.md)
  - [6.6 配置生成密钥](deploy/kubernetes-the-hard-way/06-data-encryption-keys.md)
  - [6.7 部署 Etcd 群集](deploy/kubernetes-the-hard-way/07-bootstrapping-etcd.md)
  - [6.8 部署控制节点](deploy/kubernetes-the-hard-way/08-bootstrapping-kubernetes-controllers.md)
  - [6.9 部署计算节点](deploy/kubernetes-the-hard-way/09-bootstrapping-kubernetes-workers.md)
  - [6.10 配置 Kubectl](deploy/kubernetes-the-hard-way/10-configuring-kubectl.md)
  - [6.11 配置网络路由](deploy/kubernetes-the-hard-way/11-pod-network-routes.md)
  - [6.12 部署 DNS 扩展](deploy/kubernetes-the-hard-way/12-dns-addon.md)
  - [6.13 烟雾测试](deploy/kubernetes-the-hard-way/13-smoke-test.md)
  - [6.14 删除集群](deploy/kubernetes-the-hard-way/14-cleanup.md)

## 插件扩展

- [7. 网络插件](network/index.md)
  - [网络模型](network/network.md)
  - [CNI](network/cni/index.md)
  - [Flannel](network/flannel/index.md)
  - [Calico](network/calico/index.md)
  - [Weave](network/weave/index.md)
  - [Cilium](network/cilium/index.md)
  - [OVN](network/ovn-kubernetes.md)
  - [Contiv](network/contiv/index.md)
  - [SR-IOV](network/sriov/index.md)
  - [Romana](network/romana/index.md)
  - [OpenContrail](network/opencontrail/index.md)
  - [Kuryr](network/kuryr/index.md)
- [8. 容器运行时](plugins/CRI.md)
- [9. 访问控制](plugins/auth.md)
  - [RBAC 授权](plugins/rbac.md)
  - [准入控制](plugins/admission.md)
- [10. 存储插件](plugins/volume.md)
  - [容器存储接口 CSI](plugins/csi.md)
  - [FlexVolume](plugins/flex-volume.md)
  - [glusterfs](plugins/glusterfs.md)
- [11. 网络策略](plugins/network-policy.md)
- [12. Ingress Controller](plugins/ingress.md)
  - [Traefik Ingress](practice/service-discovery-lb/service-discovery-and-load-balancing.md)
  - [minikube Ingress](practice/minikube-ingress.md)
  - [Ingress + Letsencrypt](practice/ingress_letsencrypt.md)
  - [Keepalived-VIP](plugins/keepalived-vip.md)
- [13. Cloud Provider](plugins/cloud-provider.md)
- [14. Scheduler 扩展](plugins/scheduler.md)
- [15. Device 插件](plugins/device.md)

## 服务治理

- [16. 服务治理](apps/index.md)
  - [一般准则](apps/patterns.md)
  - [滚动升级](apps/service-rolling-update.md)
  - [Helm](apps/helm.md)
  - [Operator](apps/operator.md)
  - [Service Mesh](apps/service-mesh.md)
  - [Linkerd](apps/linkerd.md)
  - [Conduit](apps/conduit.md)
  - [Istio](apps/istio.md)
    - [安装](apps/istio-deploy.md)
    - [流量管理](apps/istio-traffic-management.md)
    - [安全管理](apps/istio-security.md)
    - [策略管理](apps/istio-policy.md)
    - [Metrics](apps/istio-metrics.md)
    - [排错](apps/istio-troubleshoot.md)
    - [社区](apps/istio-community.md)
- [17. Devops](apps/devops.md)
  - [Draft](apps/draft.md)
  - [Jenkins X](apps/jenkinsx.md)
  - [Spinnaker](apps/spinnaker.md)
  - [Kompose](apps/kompose.md)
  - [Skaffold](apps/skaffold.md)
  - [Argo](apps/argo.md)
  - [Flux GitOps](apps/flux.md)

## 实践案例

- [18. 实践案例](practice/index.md)
  - [高可用](practice/ha.md)
  - [调试](practice/debugging.md)
  - [端口映射](practice/portmap.md)
  - [端口转发](practice/portforward.md)
  - [GPU](practice/gpu.md)
  - [HugePage](practice/hugepage.md)
  - [安全](practice/security.md)
  - [审计](practice/audit.md)
  - [大规模集群](practice/big-cluster.md)
- [19. 大数据与机器学习](machine-learning/index.md)
  - [Spark](machine-learning/spark.md)
  - [Tensorflow](machine-learning/tensorflow.md)
- [20. Serverless](serverless/index.md)
- [21. 排错指南](troubleshooting/index.md)
  - [集群排错](troubleshooting/cluster.md)
  - [Pod 排错](troubleshooting/pod.md)
  - [网络排错](troubleshooting/network.md)
  - [PV 排错](troubleshooting/pv.md)
    - [AzureDisk](troubleshooting/azuredisk.md)
    - [AzureFile](troubleshooting/azurefile.md)
  - [Windows 排错](troubleshooting/windows.md)
  - [云平台排错](troubleshooting/cloud.md)
    - [Azure](troubleshooting/azure.md)
  - [排错工具](troubleshooting/tools.md)

## 开发与社区贡献

- [22. 开发指南](devel/index.md)
  - [开发环境搭建](devel/index.md)
  - [单元测试和集成测试](devel/testing.md)
  - [社区贡献](devel/contribute.md)

## 附录

- [23. 附录](appendix/index.md)
  - [生态圈](appendix/ecosystem.md)
  - [学习资源](appendix/tutorial.md)
  - [如何贡献](CONTRIBUTING.md)
  - [参考文档](appendix/reference.md)
